/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package game;

import block.Block;
import block.WarpHole;
import java.awt.Image;
import javax.swing.ImageIcon;
import main.Global;
import main.SoundEffects;
import score.Score;

/**
 * Sprite is the base class for every moving object in the game.
 * @author s070280
 */
public abstract class Sprite {

    public Sprite() {
    }

    /**
     * This function changes the requested direction of Sprite, by assigning 
     * another value of the Direction enumeration to the requested direction.     * 
     * @param dir String that corresponds to a value in the Direction enumeration
     */
    public void changeDirection(String dir) {
        reqDirection = Direction.valueOf(dir.toUpperCase());
    }

    /**
     * This function checks what kind of block sprite is about to encounter. 
     * Sprite then retrieves its desired behavior from that specific block. 
     * This in done in the getEffect() function, getEffect returns true(pass) or false(stop).     
     * The check will only be executed if sprite is at the edge of a gridcell.
     * @param blocks[y][x]: input is an two dimensional array of blocks representing the current level
     */
    public void checkCollision(Block[][] blocks, Score score) {
        // First make sure sprite has made it to the edge of one gridcell
        if (coordinate[0] % Global.BLOCKSIZE.value() == 0 && coordinate[1] % Global.BLOCKSIZE.value() == 0) {
            // Get the block that is at the next gridcell where sprite's requested direction is heading and get its effec on sprite            
            if (!(blocks[(coordinate[1]) / Global.BLOCKSIZE.value() + reqDirection.getdY()][(coordinate[0]) / Global.BLOCKSIZE.value() + reqDirection.getdX()].getEffect(this, score))) {
                // If sprite cannot move into its requested direction, also checks if it can move further in its current direction
                if (!(blocks[(coordinate[1]) / Global.BLOCKSIZE.value() + direction.getdY()][(coordinate[0]) / Global.BLOCKSIZE.value() + direction.getdX()].getEffect(this, score))) {
                    speed = 0;
                } else {
                    speed = Global.SPEEDPERFRAME.value();
                    if (blocks[(coordinate[1]) / Global.BLOCKSIZE.value() + direction.getdY()][(coordinate[0]) / Global.BLOCKSIZE.value() + direction.getdX()].getTypeBlock().getSimpleName().equals("WarpHole")) {
                        WarpHole warpHole = (WarpHole) blocks[(coordinate[1]) / Global.BLOCKSIZE.value() + direction.getdY()][(coordinate[0]) / Global.BLOCKSIZE.value() + direction.getdX()];
                        Thread tsound = new Thread(new SoundEffects("warp"));
                        tsound.start();
                        warpHole.setWarp(this);
                    }
                }
            } else {
                speed = Global.SPEEDPERFRAME.value();
                // When pacman is allowed to move to its desired direction, set new direction and icon orientation
                direction = reqDirection;
                if (blocks[(coordinate[1]) / Global.BLOCKSIZE.value() + direction.getdY()][(coordinate[0]) / Global.BLOCKSIZE.value() + direction.getdX()].getTypeBlock().getSimpleName().equals("WarpHole")) {
                    WarpHole warpHole = (WarpHole) blocks[(coordinate[1]) / Global.BLOCKSIZE.value() + direction.getdY()][(coordinate[0]) / Global.BLOCKSIZE.value() + direction.getdX()];
                    Thread tsound = new Thread(new SoundEffects("warp"));
                    tsound.start();
                    warpHole.setWarp(this);
                }

            }           
        }
    }

    /**
     * Sprite directions has been calculated, now move sprite to the next coordinate.
     * Also animate sprite icon.
     */
    public void move() {
        coordinate[0] += direction.getdX() * speed * stateModifier;
        coordinate[1] += direction.getdY() * speed * stateModifier;
        //System.out.print(direction.getdX() *speed * stateModifier + "\n");

        cycleState();
        animateSprite();

    }

    /**
     * Function determines the exact icon of the sprite.
     */
    public void animateSprite() {
        // The code below determines the exact icon of the sprite.
        // Decrease the cycle time with one, when the cycletime has reached zero, 
        // the icon of the sprite and start the cycle again
        animationCycleTime--;
        if (animationCycleTime <= 0) {                              // Set the cyle to the total time that needs to elapse before the sprite icon is changed.
            animationCycleTime = animationTotalTime;
            iconNr = iconNr + animationDirection;
            setImage();
            if (iconNr == 1 || iconNr == iconCount) {               // If the animationNr has reached its first or last icon number, 
                animationDirection = -animationDirection;           // change the direction of icon numbers. (1 to 4 / 4 to 1)
            }
        }
    }

    public abstract void cycleState();

    public Integer getCoordinateX() {
        return coordinate[0];
    }

    public Integer getCoordinateY() {
        return coordinate[1];
    }

    public Image getImage() {
        return image;
    }

    /*
     * Sets the icon of a sprite to its classname and the number of the icon.
     * Each sprite has a number of icons for animating it.
     */
    public void setImage() {
        String filename = "/img/" + this.getClass().getSimpleName().toLowerCase() + iconNr.toString() + ".gif";
        ImageIcon ii = new ImageIcon(this.getClass().getResource(filename));
        image = ii.getImage();
    }

    public int getSpriteRotation() {
        return direction.getOrientation();
    }

    /*
     * Enum consisting of all four sprite directions. The arguments of each direction
     * correspond with dx, dy and sprite orientation(in degrees) in that particular order.
     */
    public enum Direction {

        RIGHT(1, 0, 0),
        LEFT(-1, 0, 180),
        UP(0, -1, 270),
        DOWN(0, 1, 90);
        private final int dx;
        private final int dy;
        private final int orientation;

        Direction(int dx, int dy, int or) {
            this.dx = dx;
            this.dy = dy;
            this.orientation = or;
        }

        public int getdX() {
            return this.dx;
        }

        public int getdY() {
            return this.dy;
        }

        public int getOrientation() {
            return this.orientation;
        }
    }
    public int animationCycleTime;
    public int animationDirection = 1;
    protected int animationTotalTime, iconCount;
    protected Integer iconNr;
    public Image image;
    public int[] coordinate = new int[2];
    public Direction direction = Direction.RIGHT;
    private Direction reqDirection = Direction.RIGHT;
    public int speed = Global.SPEEDPERFRAME.value();
    public double stateModifier = 1;
    private Block[][] currentLevel;
    public static Level level;
    boolean blinking;
}